# AUTO GENERATED FILE - DO NOT EDIT

import typing  # noqa: F401
import numbers # noqa: F401
from typing_extensions import TypedDict, NotRequired, Literal # noqa: F401
from dash.development.base_component import Component, _explicitize_args
try:
    from dash.development.base_component import ComponentType # noqa: F401
except ImportError:
    ComponentType = typing.TypeVar("ComponentType", bound=Component)


class Loading(Component):
    """A Loading component.
The Loading component makes it possible to stop event propagation during loading.

Keyword arguments:

- children (a list of or a singular dash component, string or number; optional):
    Array or single React node.

- id (string; optional):
    Unique ID to identify this component in Dash callbacks.

- loading_state (dict; optional):
    Object that holds the loading state coming from dash-renderer.

    `loading_state` is a dict with keys:

    - is_loading (boolean; optional):
        Determines if the component is loading or not.

    - prop_name (string; optional):
        Holds which property is loading.

    - component_name (string; optional):
        Holds the name of the component that is loading.

- preventDefault (list of strings; default ['keydown']):
    Events for which to call preventDefault() during loading."""
    _children_props = []
    _base_nodes = ['children']
    _namespace = 'dash_extensions'
    _type = 'Loading'
    LoadingState = TypedDict(
        "LoadingState",
            {
            "is_loading": NotRequired[bool],
            "prop_name": NotRequired[str],
            "component_name": NotRequired[str]
        }
    )

    @_explicitize_args
    def __init__(
        self,
        children: typing.Optional[typing.Union[str, int, float, ComponentType, typing.Sequence[typing.Union[str, int, float, ComponentType]]]] = None,
        id: typing.Optional[typing.Union[str, dict]] = None,
        preventDefault: typing.Optional[typing.Sequence[str]] = None,
        loading_state: typing.Optional["LoadingState"] = None,
        **kwargs
    ):
        self._prop_names = ['children', 'id', 'loading_state', 'preventDefault']
        self._valid_wildcard_attributes =            []
        self.available_properties = ['children', 'id', 'loading_state', 'preventDefault']
        self.available_wildcard_properties =            []
        _explicit_args = kwargs.pop('_explicit_args')
        _locals = locals()
        _locals.update(kwargs)  # For wildcard attrs and excess named props
        args = {k: _locals[k] for k in _explicit_args if k != 'children'}

        super(Loading, self).__init__(children=children, **args)



# AUTO GENERATED FILE - DO NOT EDIT

import typing  # noqa: F401
import numbers # noqa: F401
from typing_extensions import TypedDict, NotRequired, Literal # noqa: F401
from dash.development.base_component import Component, _explicitize_args
try:
    from dash.development.base_component import ComponentType # noqa: F401
except ImportError:
    ComponentType = typing.TypeVar("ComponentType", bound=Component)


class Keyboard(Component):
    """A Keyboard component.
A component that listens for keyboard events and forwards them to Dash.

Keyword arguments:

- children (a list of or a singular dash component, string or number; optional):
    The children of this component. If any children are provided, the
    component will listen for events from these components. If no
    children are specified, the component will listen for events from
    the document object.

- id (string; optional):
    Unique ID to identify this component in Dash callbacks.

- captureKeys (list of string | dict with strings as keys and values of type boolean | number | string | dict | lists; optional):
    The keys to capture. Defaults to all keys. Can be either a string
    (e.g. \"Enter\") or an object (e.g. {key: 'Enter', ctrlKey:
    True}).

- className (string; optional):
    A custom class name.

- eventProps (list of strings; default ["key", "altKey", "ctrlKey", "shiftKey", "metaKey", "repeat"]):
    The event properties to forward to Dash, see
    https://developer.mozilla.org/en-US/docs/Web/API/KeyboardEvent.

- keydown (dict with strings as keys and values of type boolean | number | string | dict | list; optional):
    The result of the key down event.

- keys_pressed (dict with strings as keys and values of type boolean | number | string | dict | list; optional):
    A dict of objects like keydown for all keys currently pressed.

- keyup (dict with strings as keys and values of type boolean | number | string | dict | list; optional):
    The result of the key up event.

- n_keydowns (number; default 0):
    A counter, which is incremented on each key down event.

- n_keyups (number; default 0):
    A counter, which is incremented on each key up event.

- useCapture (boolean; default False):
    Value of useCapture used when registering event listeners."""
    _children_props = []
    _base_nodes = ['children']
    _namespace = 'dash_extensions'
    _type = 'Keyboard'

    @_explicitize_args
    def __init__(
        self,
        children: typing.Optional[typing.Union[str, int, float, ComponentType, typing.Sequence[typing.Union[str, int, float, ComponentType]]]] = None,
        id: typing.Optional[typing.Union[str, dict]] = None,
        style: typing.Optional[typing.Any] = None,
        className: typing.Optional[str] = None,
        eventProps: typing.Optional[typing.Sequence[str]] = None,
        captureKeys: typing.Optional[typing.Sequence[typing.Union[str, typing.Dict[typing.Union[str, float, int], typing.Any]]]] = None,
        keydown: typing.Optional[typing.Dict[typing.Union[str, float, int], typing.Any]] = None,
        keyup: typing.Optional[typing.Dict[typing.Union[str, float, int], typing.Any]] = None,
        keys_pressed: typing.Optional[typing.Dict[typing.Union[str, float, int], typing.Any]] = None,
        n_keydowns: typing.Optional[typing.Union[int, float, numbers.Number]] = None,
        n_keyups: typing.Optional[typing.Union[int, float, numbers.Number]] = None,
        useCapture: typing.Optional[bool] = None,
        **kwargs
    ):
        self._prop_names = ['children', 'id', 'captureKeys', 'className', 'eventProps', 'keydown', 'keys_pressed', 'keyup', 'n_keydowns', 'n_keyups', 'style', 'useCapture']
        self._valid_wildcard_attributes =            []
        self.available_properties = ['children', 'id', 'captureKeys', 'className', 'eventProps', 'keydown', 'keys_pressed', 'keyup', 'n_keydowns', 'n_keyups', 'style', 'useCapture']
        self.available_wildcard_properties =            []
        _explicit_args = kwargs.pop('_explicit_args')
        _locals = locals()
        _locals.update(kwargs)  # For wildcard attrs and excess named props
        args = {k: _locals[k] for k in _explicit_args if k != 'children'}

        super(Keyboard, self).__init__(children=children, **args)



from typing import TypeVar

from dash.development.base_component import Component

ComponentIdentifier = str | dict | Component
DashNode = Component | list[Component] | str | list[str]

T = TypeVar("T")


def get_id(component: ComponentIdentifier) -> str | dict:
    if isinstance(component, Component):
        return component._set_random_id()
    return component


def as_list(item: T | list[T] | None) -> list[T]:
    if item is None:
        return []
    if isinstance(item, tuple):
        return list(item)
    if isinstance(item, list):
        return item
    return [item]



# AUTO GENERATED FILE - DO NOT EDIT

import typing  # noqa: F401
import numbers # noqa: F401
from typing_extensions import TypedDict, NotRequired, Literal # noqa: F401
from dash.development.base_component import Component, _explicitize_args
try:
    from dash.development.base_component import ComponentType # noqa: F401
except ImportError:
    ComponentType = typing.TypeVar("ComponentType", bound=Component)


class CycleBreaker(Component):
    """A CycleBreaker component.
Simple data store that automatically copies the current value of the src property into dst property. Can be used to break circular dependencies. 
*

Keyword arguments:

- id (string; optional):
    Unique ID to identify this component in Dash callbacks.

- dst (boolean | number | string | dict | list; optional):
    Read the forwarded value from this property.

- src (boolean | number | string | dict | list; optional):
    Set this property to value to be forwarded from."""
    _children_props = []
    _base_nodes = ['children']
    _namespace = 'dash_extensions'
    _type = 'CycleBreaker'

    @_explicitize_args
    def __init__(
        self,
        id: typing.Optional[typing.Union[str, dict]] = None,
        src: typing.Optional[typing.Any] = None,
        dst: typing.Optional[typing.Any] = None,
        **kwargs
    ):
        self._prop_names = ['id', 'dst', 'src']
        self._valid_wildcard_attributes =            []
        self.available_properties = ['id', 'dst', 'src']
        self.available_wildcard_properties =            []
        _explicit_args = kwargs.pop('_explicit_args')
        _locals = locals()
        _locals.update(kwargs)  # For wildcard attrs and excess named props
        args = {k: _locals[k] for k in _explicit_args}

        super(CycleBreaker, self).__init__(**args)



import datetime
from enum import Enum
import re
from typing import Optional, Type, TypeVar
from logging import getLogger
from dash import set_props, ctx

from pydantic import BaseModel, Field
from dash_extensions.enrich import Input, dcc
from dash.dependencies import DashDependency

logger = getLogger(__name__)


T = TypeVar("T", bound=DashDependency)


class EventModel(BaseModel):
    """
    The EventModel class provides the main interface for creating and dispatching events.
    """

    timestamp: float = Field(default_factory=datetime.datetime.now().timestamp)

    def add_listener(self) -> Input:
        """
        Listen for event.
        """
        return self.get_dependency(Input)

    def dispatch(self):
        """
        Dispatch event.
        """
        if self._component_id not in _event_registry:
            logger.warning(f"No listener registered for {self._uid}. Event dispatch suppressed.")
            return
        set_props(self._component_id, {"data": self.model_dump()})

    def get_dependency(self, dependency_type: Type[T]) -> T:
        """
        Get DashDependency (Input, Output, State) for event.
        """
        _register_event(self)  # TODO: Could also be on init?
        return dependency_type(self._component_id, "data")

    def is_trigger(self) -> bool:
        return self._component_id in ctx.triggered_prop_ids.values()

    @property
    def _uid(self) -> str:
        raise NotImplementedError

    @property
    def _component_id(self) -> str:
        return f"event_store_{self._uid}"


# region Simple interface (str, Enum; without payload)


class SimpleEvent(EventModel):
    """
    Wrapper class for simple events.
    """

    type: str | Enum

    @property
    def _uid(self) -> str:
        return _get_event_id(self.type)


def _get_event_id(event: str | Enum) -> str:
    if isinstance(event, Enum):
        event_tag = event.name.lower()
        event_type = _to_snake(event.__class__.__name__)
        return f"{event_type}_{event_tag}"
    return event


def _to_snake(camel: str) -> str:
    return re.sub(r"(?<!^)(?=[A-Z])", "_", camel).lower()


def _base_event(event: str | Enum) -> SimpleEvent:
    return SimpleEvent(type=event)


def dispatch_event(event: str | Enum):
    """
    Dispatch an event.
    """
    return _base_event(event).dispatch()


def get_event_dependency(event: str | Enum, dependency_type: Type[T]) -> T:
    """
    Get DashDependency (Input, Output, State) for a particular event type.
    """
    return _base_event(event).get_dependency(dependency_type)


def add_event_listener(event: str | Enum) -> Input:
    """
    Listen for a particular event type.
    """
    return _base_event(event).get_dependency(Input)


def register_event(event: str | Enum):
    """
    Register an event.
    """
    _register_event(_base_event(event))


def is_event_trigger(event: str | Enum | list[str | Enum]) -> bool:
    """
    Check if an event is a trigger of the callback.
    """
    events = [event] if not isinstance(event, list) else event
    return get_event_trigger(events) is not None


def get_event_trigger(events: list[str | Enum]) -> Optional[str | Enum]:
    """
    Get the event that triggered the callback.
    """
    for event in events:
        if _base_event(event).is_trigger():
            return event
    return None


# endregion

# region Event registry


def resolve_event_components() -> list[dcc.Store]:
    """
    Returns the components that must be included in the layout.
    """
    return [dcc.Store(id=component_id, storage_type="memory") for component_id in list(_event_registry)]


def _register_event(event: EventModel):
    _event_registry.add(event._component_id)


"""
Registry over all events that have dependencies.
"""
_event_registry: set[str] = set()

# endregion



# AUTO GENERATED FILE - DO NOT EDIT

import typing  # noqa: F401
import numbers # noqa: F401
from typing_extensions import TypedDict, NotRequired, Literal # noqa: F401
from dash.development.base_component import Component, _explicitize_args
try:
    from dash.development.base_component import ComponentType # noqa: F401
except ImportError:
    ComponentType = typing.TypeVar("ComponentType", bound=Component)


class SSE(Component):
    """A SSE component.
The SSE component makes it possible to collect data from e.g. a ResponseStream. It's a wrapper around the SSE.js library.
https://github.com/mpetazzoni/sse.js

Keyword arguments:

- id (string; optional):
    Unique ID to identify this component in Dash callbacks.

- animate_chunk (number; optional):
    Chunk size (i.e. number of characters) for the animation.

- animate_delay (number; optional):
    If set, each character is delayed by some amount of time. Used to
    animate the stream.

- animate_prefix (string; optional):
    Prefix to be excluded from the animation.

- animate_suffix (string; optional):
    Suffix to be excluded from the animation.

- animation (string; optional):
    The animation of the data.

- concat (boolean; optional):
    A boolean indicating if the stream values should be concatenated.

- done (boolean; optional):
    A boolean indicating if the (current) stream has ended.

- options (dict; optional):
    Options passed to the SSE constructor.

    `options` is a dict with keys:

    - headers (dict with strings as keys and values of type string; optional):
        - headers.

    - payload (string; optional):
        - payload as a Blob, ArrayBuffer, Dataview, FormData,
        URLSearchParams, or string.

    - method (string; optional):
        - HTTP Method.

    - withCredentials (boolean; optional):
        - flag, if credentials needed.

    - start (boolean; optional):
        - flag, if streaming should start automatically.

    - debug (boolean; optional):
        - debugging flag.

- url (string; optional):
    URL of the endpoint.

- value (string; optional):
    The data value. Either the latest, or the concatenated depending
    on the `concat` property."""
    _children_props = []
    _base_nodes = ['children']
    _namespace = 'dash_extensions'
    _type = 'SSE'
    Options = TypedDict(
        "Options",
            {
            "headers": NotRequired[typing.Dict[typing.Union[str, float, int], str]],
            "payload": NotRequired[typing.Union[str]],
            "method": NotRequired[str],
            "withCredentials": NotRequired[bool],
            "start": NotRequired[bool],
            "debug": NotRequired[bool]
        }
    )

    @_explicitize_args
    def __init__(
        self,
        id: typing.Optional[typing.Union[str, dict]] = None,
        options: typing.Optional["Options"] = None,
        url: typing.Optional[str] = None,
        concat: typing.Optional[bool] = None,
        animate_delay: typing.Optional[typing.Union[int, float, numbers.Number]] = None,
        animate_chunk: typing.Optional[typing.Union[int, float, numbers.Number]] = None,
        animate_prefix: typing.Optional[str] = None,
        animate_suffix: typing.Optional[str] = None,
        value: typing.Optional[str] = None,
        animation: typing.Optional[str] = None,
        done: typing.Optional[bool] = None,
        **kwargs
    ):
        self._prop_names = ['id', 'animate_chunk', 'animate_delay', 'animate_prefix', 'animate_suffix', 'animation', 'concat', 'done', 'options', 'url', 'value']
        self._valid_wildcard_attributes =            []
        self.available_properties = ['id', 'animate_chunk', 'animate_delay', 'animate_prefix', 'animate_suffix', 'animation', 'concat', 'done', 'options', 'url', 'value']
        self.available_wildcard_properties =            []
        _explicit_args = kwargs.pop('_explicit_args')
        _locals = locals()
        _locals.update(kwargs)  # For wildcard attrs and excess named props
        args = {k: _locals[k] for k in _explicit_args}

        super(SSE, self).__init__(**args)



# AUTO GENERATED FILE - DO NOT EDIT

import typing  # noqa: F401
import numbers # noqa: F401
from typing_extensions import TypedDict, NotRequired, Literal # noqa: F401
from dash.development.base_component import Component, _explicitize_args
try:
    from dash.development.base_component import ComponentType # noqa: F401
except ImportError:
    ComponentType = typing.TypeVar("ComponentType", bound=Component)


class BeforeAfter(Component):
    """A BeforeAfter component.
BeforeAfter — A before‑and‑after image slider built on img-comparison-slider.
*

Keyword arguments:

- id (string; optional):
    Unique ID to identify this component in Dash callbacks.

- after (dict; optional):
    Props for the `after` Img component. eg {\"src\":
    \"/assets/lena_color.png\"}.

- before (dict; optional):
    Props for the `before` Img component. eg {\"src\":
    \"/assets/lena_bw.png\"}.

- direction (a value equal to: 'horizontal', 'vertical'; optional):
    Set slider direction.

- height (string; optional):
    Image height — default \"auto\" for responsive images.

- hover (boolean; optional):
    Automatic slide on mouse over.

- keyboard (a value equal to: 'enabled', 'disabled'; optional):
    Enable/disable slider position control with the keyboard.

- value (number; optional):
    The divider position can be specified as a percentage, i.e. 0 to
    100.

- width (string; optional):
    Image width — default \"100%\" for responsive images."""
    _children_props = []
    _base_nodes = ['children']
    _namespace = 'dash_extensions'
    _type = 'BeforeAfter'

    @_explicitize_args
    def __init__(
        self,
        id: typing.Optional[typing.Union[str, dict]] = None,
        height: typing.Optional[str] = None,
        width: typing.Optional[str] = None,
        hover: typing.Optional[bool] = None,
        value: typing.Optional[typing.Union[int, float, numbers.Number]] = None,
        direction: typing.Optional[Literal["horizontal", "vertical"]] = None,
        keyboard: typing.Optional[Literal["enabled", "disabled"]] = None,
        before: typing.Optional[dict] = None,
        after: typing.Optional[dict] = None,
        **kwargs
    ):
        self._prop_names = ['id', 'after', 'before', 'direction', 'height', 'hover', 'keyboard', 'value', 'width']
        self._valid_wildcard_attributes =            []
        self.available_properties = ['id', 'after', 'before', 'direction', 'height', 'hover', 'keyboard', 'value', 'width']
        self.available_wildcard_properties =            []
        _explicit_args = kwargs.pop('_explicit_args')
        _locals = locals()
        _locals.update(kwargs)  # For wildcard attrs and excess named props
        args = {k: _locals[k] for k in _explicit_args}

        super(BeforeAfter, self).__init__(**args)



# AUTO GENERATED FILE - DO NOT EDIT

import typing  # noqa: F401
import numbers # noqa: F401
from typing_extensions import TypedDict, NotRequired, Literal # noqa: F401
from dash.development.base_component import Component, _explicitize_args
try:
    from dash.development.base_component import ComponentType # noqa: F401
except ImportError:
    ComponentType = typing.TypeVar("ComponentType", bound=Component)


class EventListener(Component):
    """An EventListener component.
A component that listens for events and forwards them to Dash.

Keyword arguments:

- children (a list of or a singular dash component, string or number; optional):
    The children of this component. If any children are provided, the
    component will listen for events from these components. If no
    children are specified, the component will listen for events from
    the document object.

- id (string; optional):
    Unique ID to identify this component in Dash callbacks.

- className (string; optional):
    A custom class name.

- event (dict; optional):
    The latest event fired.

- events (list of dicts; default [    {      event: 'keydown',      props: ['key', 'altKey', 'ctrlKey', 'shiftKey', 'metaKey', 'repeat']    }  ]):
    The event entry specifies which event to listen to, e.g. \"click\"
    for click events. The \"props\" entry specifies what event
    properties to record, e.g. [\"x\", \"y\"] to get the cursor
    position.

    `events` is a list of dicts with keys:

    - event (string; required)

    - props (list of strings; optional)

- logging (boolean; default False):
    If True, event information is logged to the javascript console.

- n_events (number; default 0):
    The number of events fired.

- useCapture (boolean; default False):
    Value of useCapture used when registering event listeners."""
    _children_props = []
    _base_nodes = ['children']
    _namespace = 'dash_extensions'
    _type = 'EventListener'
    Events = TypedDict(
        "Events",
            {
            "event": str,
            "props": NotRequired[typing.Sequence[str]]
        }
    )

    @_explicitize_args
    def __init__(
        self,
        children: typing.Optional[typing.Union[str, int, float, ComponentType, typing.Sequence[typing.Union[str, int, float, ComponentType]]]] = None,
        id: typing.Optional[typing.Union[str, dict]] = None,
        events: typing.Optional[typing.Sequence["Events"]] = None,
        logging: typing.Optional[bool] = None,
        style: typing.Optional[typing.Any] = None,
        className: typing.Optional[str] = None,
        event: typing.Optional[dict] = None,
        n_events: typing.Optional[typing.Union[int, float, numbers.Number]] = None,
        useCapture: typing.Optional[bool] = None,
        **kwargs
    ):
        self._prop_names = ['children', 'id', 'className', 'event', 'events', 'logging', 'n_events', 'style', 'useCapture']
        self._valid_wildcard_attributes =            []
        self.available_properties = ['children', 'id', 'className', 'event', 'events', 'logging', 'n_events', 'style', 'useCapture']
        self.available_wildcard_properties =            []
        _explicit_args = kwargs.pop('_explicit_args')
        _locals = locals()
        _locals.update(kwargs)  # For wildcard attrs and excess named props
        args = {k: _locals[k] for k in _explicit_args if k != 'children'}

        super(EventListener, self).__init__(children=children, **args)



# AUTO GENERATED FILE - DO NOT EDIT

import typing  # noqa: F401
import numbers # noqa: F401
from typing_extensions import TypedDict, NotRequired, Literal # noqa: F401
from dash.development.base_component import Component, _explicitize_args
try:
    from dash.development.base_component import ComponentType # noqa: F401
except ImportError:
    ComponentType = typing.TypeVar("ComponentType", bound=Component)


class Lottie(Component):
    """A Lottie component.
Light wrapper of the lottie-react component for Dash.

Keyword arguments:

- id (string; optional):
    Unique ID to identify this component in Dash callbacks.

- action (a value equal to: 'play', 'pause', 'stop'; optional):
    Actions routed to the Lottie component to control the animation
    state.

- options (dict; optional):
    Options passed to the Lottie animation (see
    https://github.com/Gamote/lottie-react for details).

- speed (number; optional):
    Animation speed. 1 is normal speed (and default).

- url (string; optional):
    If set, data will be downloaded from this URL."""
    _children_props = []
    _base_nodes = ['children']
    _namespace = 'dash_extensions'
    _type = 'Lottie'

    @_explicitize_args
    def __init__(
        self,
        id: typing.Optional[typing.Union[str, dict]] = None,
        url: typing.Optional[str] = None,
        speed: typing.Optional[typing.Union[int, float, numbers.Number]] = None,
        action: typing.Optional[Literal["play", "pause", "stop"]] = None,
        options: typing.Optional[dict] = None,
        **kwargs
    ):
        self._prop_names = ['id', 'action', 'options', 'speed', 'url']
        self._valid_wildcard_attributes =            []
        self.available_properties = ['id', 'action', 'options', 'speed', 'url']
        self.available_wildcard_properties =            []
        _explicit_args = kwargs.pop('_explicit_args')
        _locals = locals()
        _locals.update(kwargs)  # For wildcard attrs and excess named props
        args = {k: _locals[k] for k in _explicit_args}

        super(Lottie, self).__init__(**args)



# AUTO GENERATED FILE - DO NOT EDIT

import typing  # noqa: F401
import numbers # noqa: F401
from typing_extensions import TypedDict, NotRequired, Literal # noqa: F401
from dash.development.base_component import Component, _explicitize_args
try:
    from dash.development.base_component import ComponentType # noqa: F401
except ImportError:
    ComponentType = typing.TypeVar("ComponentType", bound=Component)


class Mermaid(Component):
    """A Mermaid component.
Light wrapper of the react-lottie component for Dash.

Keyword arguments:

- id (string; optional):
    Unique ID to identify this component in Dash callbacks.

- chart (string; optional):
    The mermaid code of your chart. Check Mermaid js documentation for
    details.

- className (string; optional):
    The class of the component.

- config (dict with strings as keys and values of type boolean | number | string | dict | list; optional):
    An optional object with one of several Mermaid config parameters.
    Check Mermaid js documentation for details.

- name (string; optional):
    An optional name of your mermaid diagram/flowchart/gantt etc."""
    _children_props = []
    _base_nodes = ['children']
    _namespace = 'dash_extensions'
    _type = 'Mermaid'

    @_explicitize_args
    def __init__(
        self,
        id: typing.Optional[typing.Union[str, dict]] = None,
        chart: typing.Optional[str] = None,
        name: typing.Optional[str] = None,
        config: typing.Optional[typing.Dict[typing.Union[str, float, int], typing.Any]] = None,
        className: typing.Optional[str] = None,
        **kwargs
    ):
        self._prop_names = ['id', 'chart', 'className', 'config', 'name']
        self._valid_wildcard_attributes =            []
        self.available_properties = ['id', 'chart', 'className', 'config', 'name']
        self.available_wildcard_properties =            []
        _explicit_args = kwargs.pop('_explicit_args')
        _locals = locals()
        _locals.update(kwargs)  # For wildcard attrs and excess named props
        args = {k: _locals[k] for k in _explicit_args}

        super(Mermaid, self).__init__(**args)



from __future__ import print_function as _

import json
import os as _os
import sys as _sys

import dash as _dash

# noinspection PyUnresolvedReferences
from ._imports_ import *  # type: ignore # noqa: F401, F403
from ._imports_ import __all__  # type: ignore

if not hasattr(_dash, "__plotly_dash") and not hasattr(_dash, "development"):
    print(
        "Dash was not successfully imported. "
        "Make sure you don't have a file "
        'named \n"dash.py" in your current directory.',
        file=_sys.stderr,
    )
    _sys.exit(1)

_basepath = _os.path.dirname(__file__)
_filepath = _os.path.abspath(_os.path.join(_basepath, "package-info.json"))
with open(_filepath) as f:
    package = json.load(f)

package_name = package["name"].replace(" ", "_").replace("-", "_")
__version__ = package["version"]

_current_path = _os.path.dirname(_os.path.abspath(__file__))

_this_module = _sys.modules[__name__]

# Add async components here.
async_resources = [
    "BeforeAfter",
    "Lottie",
    "Mermaid",
    "Purify",
    "SSE",
    "null",  # part of Mermaid
]
async_chunks = [f"async-{async_resource}" for async_resource in async_resources]

# Add shared chunks here.
shared_chunks = []

# Collect all chunks (main, async, shared).
chunks = [__name__] + async_chunks + shared_chunks

# Add all chunks to the js_dist list.
_js_dist = []
_js_dist.extend(
    [
        {
            "relative_package_path": f"{chunk}.js",
            "external_url": f"https://unpkg.com/{package_name}@{__version__}/{__name__}/{chunk}.js",
            "namespace": package_name,
            "async": chunk != __name__,  # don't make the main bundle async
        }
        for chunk in chunks
    ]
)
_js_dist.extend(
    [
        {
            "relative_package_path": f"{chunk}.js.map",
            "external_url": f"https://unpkg.com/{package_name}@{__version__}/{__name__}/{chunk}.js.map",
            "namespace": package_name,
            "dynamic": True,
        }
        for chunk in chunks
    ]
)

# Enable runtime prop types validation with tsx components.
# _js_dist.append(dict(dev_package_path="proptypes.js", namespace="dash_extensions"))

_css_dist = []


for _component in __all__:
    setattr(locals()[_component], "_js_dist", _js_dist)
    setattr(locals()[_component], "_css_dist", _css_dist)



# AUTO GENERATED FILE - DO NOT EDIT

import typing  # noqa: F401
import numbers # noqa: F401
from typing_extensions import TypedDict, NotRequired, Literal # noqa: F401
from dash.development.base_component import Component, _explicitize_args
try:
    from dash.development.base_component import ComponentType # noqa: F401
except ImportError:
    ComponentType = typing.TypeVar("ComponentType", bound=Component)


class WebSocket(Component):
    """A WebSocket component.
A simple interface to the WebSocket API.

Keyword arguments:

- id (string; optional):
    Unique ID to identify this component in Dash callbacks.

- error (boolean | number | string | dict | list; optional):
    This property is set with the content of the onerror event.

- message (boolean | number | string | dict | list; optional):
    When messages are received, this property is updated with the
    message content.

- protocols (list of strings; optional):
    Supported websocket protocols (optional).

- send (boolean | number | string | dict | list; optional):
    When this property is set, a message is sent with its content.

- state (boolean | number | string | dict | list; default { readyState: WebSocket.CLOSED }):
    This websocket state (in the readyState prop) and associated
    information.

- timeout (number; default 1000):
    How many ms to wait for websocket to be ready when sending a
    message (optional).

- url (string; optional):
    The websocket endpoint (e.g. wss://echo.websocket.org)."""
    _children_props = []
    _base_nodes = ['children']
    _namespace = 'dash_extensions'
    _type = 'WebSocket'

    @_explicitize_args
    def __init__(
        self,
        id: typing.Optional[typing.Union[str, dict]] = None,
        state: typing.Optional[typing.Any] = None,
        message: typing.Optional[typing.Any] = None,
        error: typing.Optional[typing.Any] = None,
        send: typing.Optional[typing.Any] = None,
        url: typing.Optional[str] = None,
        protocols: typing.Optional[typing.Sequence[str]] = None,
        timeout: typing.Optional[typing.Union[int, float, numbers.Number]] = None,
        **kwargs
    ):
        self._prop_names = ['id', 'error', 'message', 'protocols', 'send', 'state', 'timeout', 'url']
        self._valid_wildcard_attributes =            []
        self.available_properties = ['id', 'error', 'message', 'protocols', 'send', 'state', 'timeout', 'url']
        self.available_wildcard_properties =            []
        _explicit_args = kwargs.pop('_explicit_args')
        _locals = locals()
        _locals.update(kwargs)  # For wildcard attrs and excess named props
        args = {k: _locals[k] for k in _explicit_args}

        super(WebSocket, self).__init__(**args)



from __future__ import annotations

import logging
import uuid
from typing import Callable, Dict

import dash
from dash import (
    Output,
    html,
)
from dash._callback_context import _get_context_value, has_context
from dash._utils import stringify_id

from dash_extensions.utils import Component, DashNode, as_list


@has_context
def set_props(component_id: str | dict, props: dict, append: bool = False) -> None:
    """
    In the current (upstream) implementation of Dash, the "set_props" function overrides the value on each call,
    effectively leaving only the last value in the updated_props dictionary. This implementation allows for *appending*.
    """
    ctx_value = _get_context_value()
    _id = stringify_id(component_id)
    if not append or _id not in ctx_value.updated_props:
        ctx_value.updated_props[_id] = props
        return
    updated_props = dict(ctx_value.updated_props[_id]) if _id in ctx_value.updated_props else {}
    for key in props:
        if key not in updated_props:
            updated_props[key] = props[key]
            continue
        updated = as_list(updated_props[key])
        updated.append(props[key])
        updated_props[key] = updated
    ctx_value.updated_props[_id] = updated_props


class DashLogHandler(logging.Handler):
    """
    DashLogHandler is a logging handler that writes log messages to a Dash component. It fits inside the existing
    logging framework of Python, while providing the ability to update Dash UI components.
    """

    def __init__(
        self,
        output: Output,
        log_writers: Dict[int, Callable],
        layout: Component | list[Component] | None = None,
        level=logging.DEBUG,
    ):
        self.output = output
        self.log_writers = log_writers
        self.layout = layout
        # Per default set the level to DEBUG (i.e. log it all).
        logging.Handler.__init__(self=self, level=level)

    def emit(self, record):
        """
        The emit method calls set_props to update the (Dash) log output component when logs are written.
        """
        # If we are in a Dash callback context, we update the log output.
        try:
            if record.levelno in self.log_writers:
                msg = self.log_writers[record.levelno](record.getMessage())
                set_props(self.output.component_id, {self.output.component_property: msg}, append=True)
        # If we are not in a Dash callback context, we the log invocation.
        except dash.exceptions.MissingCallbackContextException:
            pass

    def embed(self) -> list[Component]:
        """
        Return any components that should be embedded in the app layout.
        """
        if self.layout is None:
            return []
        return as_list(self.layout)

    def setup_logger(self, logger_name: str = "dash_extensions", level: int = logging.DEBUG) -> logging.Logger:
        """
        Convenience method to set up a logger with this handler.
        """
        logger = logging.getLogger(logger_name)
        logger.addHandler(self)
        logger.setLevel(level)
        return logger


def get_default_log_writers() -> dict[int, Callable]:
    """
    Simplest possible log writers for INFO, WARNING, and ERROR messages.
    """
    return {
        logging.INFO: lambda x, **kwargs: html.Div(f"INFO: {x}", **kwargs),
        logging.WARNING: lambda x, **kwargs: html.Div(f"WARNING: {x}", **kwargs),
        logging.ERROR: lambda x, **kwargs: html.Div(f"ERROR: {x}", **kwargs),
    }


class DivLogHandler(DashLogHandler):
    """
    Simplest possible DashLogHandler. Just writes log messages to a Div component.
    """

    def __init__(self, log_div: DashNode | None = None) -> None:
        log_div = html.Div(id="div_log_handler") if log_div is None else log_div
        super().__init__(output=Output(log_div, "children"), log_writers=get_default_log_writers(), layout=log_div)


def get_notification_log_writers() -> dict[int, Callable]:
    """
    Log writers that target the Notification component from dash_mantine_components.
    """
    import dash_mantine_components as dmc

    def _default_kwargs(color, title, message):
        return dict(
            color=color,
            title=title,
            message=message,
            id=str(uuid.uuid4()),
            action="show",
            autoClose=False,
        )

    def log_info(message, **kwargs):
        return dmc.Notification(**{**_default_kwargs("blue", "Info", message), **kwargs})

    def log_warning(message, **kwargs):
        return dmc.Notification(**{**_default_kwargs("yellow", "Warning", message), **kwargs})

    def log_error(message, **kwargs):
        return dmc.Notification(**{**_default_kwargs("red", "Error", message), **kwargs})

    return {
        logging.INFO: log_info,
        logging.WARNING: log_warning,
        logging.ERROR: log_error,
    }


class NotificationsLogHandler(DashLogHandler):
    """
    Dash log handler that displays the logs as notifications using dash_mantine_components.
    """

    def __init__(self, log_div: DashNode | None = None, notifications_provider: DashNode | None = None) -> None:
        log_div = html.Div(id="notifications_log_handler") if log_div is None else log_div
        if notifications_provider is None:
            import dash_mantine_components as dmc

            notifications_provider = (
                dmc.NotificationsProvider() if dmc.__version__ < "0.14.0" else dmc.NotificationProvider()  # type: ignore
            )
        super().__init__(
            output=Output(log_div, "children"),
            log_writers=get_notification_log_writers(),
            layout=html.Div([notifications_provider, log_div]),  # type: ignore
        )
